@@ -1,3 +1,6 @@
+# Copyright 2024 Robert Eckelmann
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 # Configuration file for the Sphinx documentation builder.
 #
 # For the full list of built-in configuration values, see the documentation:
@@ -55,13 +58,17 @@ html_theme_options: dict[str, Any] = {
 }
 
 # Output file base name for HTML help builder.
-htmlhelp_basename = "pmboostrapdoc"
+htmlhelp_basename = "pmbootstrapdoc"
 
 html_theme_options = {
     "display_version": True,
     "style_external_links": True,
 }
 
+# customize styling
+html_static_path = ["_static"]
+html_css_files = ["pmOS.css"]
+
 # -- Options for manual page output ---------------------------------------
 
 # One entry per manual page. List of tuples
