@@ -8,9 +8,11 @@ from pmb.core.pkgrepo import pkgrepo_default_path
 from pmb.helpers import logging
 from pmb.helpers.exceptions import NonBugError
 import glob
+import heapq
 import json
 import os
 import shutil
+import urllib
 from pathlib import Path
 from typing import Any
 
@@ -102,8 +104,7 @@ def ask_for_work_path(default: Path | None) -> tuple[Path, bool]:
 
     :returns: (path, exists)
         * path: is the full path, with expanded ~ sign
-        * exists: is False when the folder did not exist before we tested whether we can create it
-
+        * exists: is True when the folder existed before we tested whether we can create it
     """
     logging.info(
         "Location of the 'work' path. Multiple chroots"
@@ -112,7 +113,11 @@ def ask_for_work_path(default: Path | None) -> tuple[Path, bool]:
     )
     while True:
         try:
-            work = os.path.expanduser(pmb.helpers.cli.ask("Work path", None, default, False))
+            work = os.path.expanduser(
+                pmb.helpers.cli.ask(
+                    "Work path", None, str(default) if default is not None else default, False
+                )
+            )
             work = os.path.realpath(work)
             exists = os.path.exists(work)
 
@@ -140,7 +145,46 @@ def ask_for_work_path(default: Path | None) -> tuple[Path, bool]:
             # Create cache_git dir, so it is owned by the host system's user
             # (otherwise pmb.helpers.mount.bind would create it as root)
             os.makedirs(f"{work}/cache_git", 0o700, True)
-            return (work, exists)
+            return (Path(work), exists)
+        except OSError:
+            logging.fatal(
+                "ERROR: Could not create this folder, or write inside it! Please try again."
+            )
+
+
+def ask_for_pmaports_path(default: Path) -> Path:
+    """Ask for a path for pmaports, until it can either be created or already exists.
+
+    :returns: full path to the directory, with a leading ~ expanded.
+    """
+    logging.info("Location of the 'pmaports' path, containing package definitions.")
+    while True:
+        try:
+            raw = os.path.expanduser(
+                pmb.helpers.cli.ask(
+                    "pmaports path",
+                    None,
+                    str(default),
+                    False,
+                )
+            )
+            path = Path(raw).resolve()
+
+            if path.is_relative_to(pmb.config.pmb_src):
+                logging.error(
+                    "ERROR: The pmaports path must not be inside the"
+                    " pmbootstrap path. Please specify another location."
+                )
+                continue
+
+            if not path.exists():
+                try:
+                    path.parent.mkdir(0o700, parents=True, exist_ok=True)
+                except Exception as e:
+                    logging.error("ERROR: Could not create parent directories: %s", str(e))
+                    continue
+
+            return path
         except OSError:
             logging.fatal(
                 "ERROR: Could not create this folder, or write inside it! Please try again."
@@ -170,7 +214,7 @@ def ask_for_channel(config: Config) -> str:
     # Otherwise, if valid: channel from pmaports.cfg of current branch
     # The actual channel name is not saved in pmbootstrap_v3.cfg, because then
     # we would need to sync it with what is checked out in pmaports.git.
-    default = pmb.config.pmaports.read_config()["channel"]
+    default = pmb.config.pmaports.read_config(add_systemd_prefix=False)["channel"]
     choices = channels_cfg["channels"].keys()
     if config.is_default_channel or default not in choices:
         default = channels_cfg["meta"]["recommended"]
@@ -186,12 +230,13 @@ def ask_for_channel(config: Config) -> str:
 def ask_for_ui(deviceinfo: Deviceinfo) -> str:
     ui_list = pmb.helpers.ui.list_ui(deviceinfo.arch)
     hidden_ui_count = 0
-    device_is_accelerated = deviceinfo.gpu_accelerated == "true"
-    if not device_is_accelerated:
+    if not deviceinfo.drm == "true":
         for i in reversed(range(len(ui_list))):
             pkgname = f"postmarketos-ui-{ui_list[i][0]}"
             apkbuild = pmb.helpers.pmaports.get(pkgname, subpackages=False, must_exist=False)
-            if apkbuild and "pmb:gpu-accel" in apkbuild["options"]:
+            if apkbuild and (
+                "pmb:drm" in apkbuild["options"] or "pmb:gpu-accel" in apkbuild["options"]
+            ):
                 ui_list.pop(i)
                 hidden_ui_count += 1
 
@@ -208,7 +253,7 @@ def ask_for_ui(deviceinfo: Deviceinfo) -> str:
     if hidden_ui_count > 0:
         logging.info(
             f"NOTE: {hidden_ui_count} UIs are hidden because"
-            ' "deviceinfo_gpu_accelerated" is not set (see'
+            ' "deviceinfo_drm" is not set (see'
             " https://postmarketos.org/deviceinfo)."
         )
     while True:
@@ -233,7 +278,7 @@ def ask_for_ui_extras(config: Config, ui: str) -> bool:
     if extra is None:
         return False
 
-    logging.info("This user interface has an extra package:" f" {extra['pkgdesc']}")
+    logging.info(f"This user interface has an extra package: {extra['pkgdesc']}")
 
     return pmb.helpers.cli.confirm("Enable this package?", default=config.ui_extras)
 
@@ -259,18 +304,18 @@ def ask_for_systemd(config: Config, ui: str) -> SystemdConfig:
     answer = pmb.helpers.cli.ask(
         "Install systemd?",
         choices,
-        config.systemd,
+        str(config.systemd),
         validation_regex=f"^({'|'.join(choices)})$",
         complete=choices,
     )
-    return answer
+    return SystemdConfig(answer)
 
 
 def ask_for_keymaps(config: Config, deviceinfo: Deviceinfo) -> str:
     if not deviceinfo.keymaps or deviceinfo.keymaps.strip() == "":
         return ""
     options = deviceinfo.keymaps.split(" ")
-    logging.info(f"Available keymaps for device ({len(options)}): " f"{', '.join(options)}")
+    logging.info(f"Available keymaps for device ({len(options)}): {', '.join(options)}")
     if config.keymap == "":
         config.keymap = options[0]
 
@@ -336,7 +381,7 @@ def ask_for_provider_select(apkbuild: dict[str, Any], providers_cfg: dict[str, s
                 # Display as default provider
                 styles = pmb.config.styles
                 logging.info(
-                    f"* {short}: {pkg['pkgdesc']} " f"{styles['BOLD']}(default){styles['END']}"
+                    f"* {short}: {pkg['pkgdesc']} {styles['BOLD']}(default){styles['END']}"
                 )
                 has_default = True
             else:
@@ -351,8 +396,7 @@ def ask_for_provider_select(apkbuild: dict[str, Any], providers_cfg: dict[str, s
                 # Selecting default means to not select any provider explicitly
                 # In other words, apk chooses it automatically based on
                 # "provider_priority"
-                if select in providers_cfg:
-                    del providers_cfg[select]
+                providers_cfg.pop(select, None)
                 break
             if ret in providers_short:
                 providers_cfg[select] = providers_short[ret]
@@ -395,7 +439,7 @@ def ask_for_device_kernel(config: Config, device: str) -> str:
     # Get default
     default = config.kernel
     if default not in kernels:
-        default = list(kernels.keys())[0]
+        default = next(iter(kernels.keys()))
 
     # Ask for kernel (extra message when downstream and upstream are available)
     logging.info("Which kernel do you want to use with your device?")
@@ -403,9 +447,8 @@ def ask_for_device_kernel(config: Config, device: str) -> str:
         logging.info("Downstream kernels are typically the outdated Android kernel forks.")
     if "downstream" in kernels and len(kernels) > 1:
         logging.info(
-            "Upstream kernels (mainline, stable, ...) get security"
-            " updates, but may have less working features than"
-            " downstream kernels."
+            "Upstream kernels (mainline, stable, ...) get security updates, but may have less"
+            " working features than downstream kernels."
         )
 
     # list kernels
@@ -448,32 +491,45 @@ def ask_for_device(context: Context) -> tuple[str, bool, str]:
         codenames = []
         if new_vendor:
             logging.info(
-                f"The specified vendor ({vendor}) could not be found in"
-                " existing ports, do you want to start a new"
-                " port?"
+                f"The specified vendor ({vendor}) could not be found in existing ports, do you want"
+                " to start a new port?"
             )
             if not pmb.helpers.cli.confirm(default=True):
                 continue
         else:
-            # Archived devices can be selected, but are not displayed
-            devices = sorted(pmb.helpers.devices.list_codenames(vendor, archived=False))
-            # Remove "vendor-" prefixes from device list
-            codenames = [x.split("-", 1)[1] for x in devices]
-            logging.info(f"Available codenames ({len(codenames)}): " + ", ".join(codenames))
+            device_list = "Devices are categorised as follows, from best to worst:\n"
+            styles = pmb.config.styles
+            for category in pmb.helpers.devices.DeviceCategory.shown():
+                device_list += f"* {category.color()}{str(category).capitalize()}{styles['END']}: {category.explain()}.\n"
+            device_entries = pmb.helpers.devices.list_codenames(vendor)
+            # Sort devices alphabetically.
+            device_entries = sorted(
+                device_entries, key=pmb.helpers.devices.DeviceEntry.codename_without_vendor
+            )
+            device_count = len(device_entries)
+            device_list += f"\nAvailable devices by codename ({device_count}): "
+            device_strings = []
+            for device_entry in device_entries:
+                codenames.append(device_entry.codename_without_vendor())
+                device_strings.append(str(device_entry))
+
+            device_list += ", ".join(device_strings)
+            logging.info(device_list)
 
         if current_vendor != vendor:
             current_codename = ""
         codename = pmb.helpers.cli.ask(
-            "Device codename", None, current_codename, False, r"[a-z0-9]+", codenames
+            "Device codename", None, current_codename, False, r"[a-z0-9\._-]+", codenames
         )
 
         device = f"{vendor}-{codename}"
         device_path = pmb.helpers.devices.find_path(device, "deviceinfo")
+        if device_path is not None:
+            device_category = pmb.helpers.devices.get_device_category_by_apkbuild_path(device_path)
         if device_path is None:
             if device == context.config.device:
                 raise RuntimeError(
-                    "This device does not exist anymore, check"
-                    " <https://postmarketos.org/renamed>"
+                    "This device does not exist anymore, check <https://postmarketos.org/renamed>"
                     " to see if it was renamed"
                 )
             logging.info(f"You are about to do a new device port for '{device}'.")
@@ -483,14 +539,20 @@ def ask_for_device(context: Context) -> tuple[str, bool, str]:
 
             # New port creation confirmed
             logging.info(f"Generating new aports for: {device}...")
-            pmb.aportgen.generate(f"device-{device}", False)
-            pmb.aportgen.generate(f"linux-{device}", False)
-        elif any("archived" == x for x in device_path.parts):
+            pmb.aportgen.generate(f"device-{device}")
+            pmb.aportgen.generate(f"linux-{device}")
+        elif device_category == pmb.helpers.devices.DeviceCategory.ARCHIVED:
             apkbuild = device_path.parent / "APKBUILD"
-            archived = pmb.parse._apkbuild.archived(apkbuild)
+            archived = pmb.parse._apkbuild.archived(apkbuild) or "No reason given (this is a bug)"
             logging.info(f"WARNING: {device} is archived: {archived}")
             if not pmb.helpers.cli.confirm():
                 continue
+        elif device_category == pmb.helpers.devices.DeviceCategory.DOWNSTREAM:
+            logging.info(
+                f"WARNING: {device} is a downstream port! Expect missing and broken features. See https://wiki.postmarketos.org/wiki/Downstream_kernel_specific_package"
+            )
+            if not pmb.helpers.cli.confirm():
+                continue
         break
 
     kernel = ask_for_device_kernel(context.config, device)
@@ -515,18 +577,17 @@ def ask_for_additional_options(config: Config) -> None:
 
     # Extra space
     logging.info(
-        "Set extra free space to 0, unless you ran into a 'No space"
-        " left on device' error. In that case, the size of the"
-        " rootfs could not be calculated properly on your machine,"
-        " and we need to add extra free space to make the image big"
-        " enough to fit the rootfs (pmbootstrap#1904)."
-        " How much extra free space do you want to add to the image"
-        " (in MB)?"
+        "Set extra free space to 0, unless you ran into a 'No space left on device' error. In that"
+        " case, the size of the rootfs could not be calculated properly on your machine, and we"
+        " need to add extra free space to make the image big enough to fit the rootfs"
+        " (pmbootstrap#1904). How much extra free space do you want to add to the image (in MB)?"
     )
+    # TODO: The __setattr__ implementation in Config does handle the conversions here,
+    # but mypy doesn't understand this (yet?), so we have to do it explicitly.
     answer = pmb.helpers.cli.ask(
         "Extra space size", None, config.extra_space, validation_regex="^[0-9]+$"
     )
-    config.extra_space = answer
+    config.extra_space = int(answer)
 
     # Boot size
     logging.info("What should be the boot partition size (in MB)?")
@@ -542,11 +603,9 @@ def ask_for_additional_options(config: Config) -> None:
 
     # Ccache size
     logging.info(
-        "We use ccache to speed up building the same code multiple"
-        " times. How much space should the ccache folder take up per"
-        " architecture? After init is through, you can check the"
-        " current usage with 'pmbootstrap stats'. Answer with 0 for"
-        " infinite."
+        "We use ccache to speed up building the same code multiple times. How much space should the"
+        " ccache folder take up per architecture? After init is through, you can check the current"
+        " usage with 'pmbootstrap stats'. Answer with 0 for infinite."
     )
     regex = "0|[0-9]+(k|M|G|T|Ki|Mi|Gi|Ti)"
     answer = pmb.helpers.cli.ask(
@@ -556,33 +615,28 @@ def ask_for_additional_options(config: Config) -> None:
 
     # Sudo timer
     logging.info(
-        "pmbootstrap does everything in Alpine Linux chroots, so"
-        " your host system does not get modified. In order to"
-        " work with these chroots, pmbootstrap calls 'sudo'"
-        " internally. For long running operations, it is possible"
-        " that you'll have to authorize sudo more than once."
+        "pmbootstrap does everything in Alpine Linux chroots, so your host system does not get"
+        " modified. In order to work with these chroots, pmbootstrap calls 'sudo' internally. For"
+        " long running operations, it is possible that you'll have to authorize sudo more than"
+        " once."
     )
-    answer = pmb.helpers.cli.confirm(
+    answer_background_timer = pmb.helpers.cli.confirm(
         "Enable background timer to prevent repeated sudo authorization?",
         default=context.sudo_timer,
     )
-    config.sudo_timer = answer
+    config.sudo_timer = answer_background_timer
 
     # Mirrors
     # prompt for mirror change
-    logging.info("Selected mirror:" f" {context.config.mirrors['pmaports']}")
+    logging.info(f"Selected mirror: {context.config.mirrors['pmaports']}")
     if pmb.helpers.cli.confirm("Change mirror?", default=False):
         mirror = ask_for_mirror()
         config.mirrors["pmaports"] = mirror
-        # FIXME: this path will change once the systemd repository is
-        # integrated into bpo (fixing this is a tasks in bpo#140)
-        # FIXME: when updating this, also add 'pmbootstrap config mirrors.systemd'
-        # examples to docs/mirrors.md
-        config.mirrors["systemd"] = os.path.join(mirror, "staging/systemd/")
+        config.mirrors["systemd"] = os.path.join(mirror, "extra-repos/systemd/")
 
 
 def ask_for_mirror() -> str:
-    regex = "^[1-9][0-9]*$"  # single non-zero number only
+    regex = "^(?:[1-9][0-9]*|best)$"  # single non-zero number only
 
     json_path = pmb.helpers.http.download(
         "https://postmarketos.org/mirrors.json", "pmos_mirrors", cache=False
@@ -597,6 +651,7 @@ def ask_for_mirror() -> str:
     for key in keys:
         logging.info(f"[{i}]\t{key} ({mirrors[key]['location']})")
         i += 1
+    logging.info("choose 'best' to select the one closest to you")
 
     urls = []
     for key in keys:
@@ -615,23 +670,40 @@ def ask_for_mirror() -> str:
         if len(link_list) > 0:
             urls.append(link_list[0])
 
-    mirror_indexes = []
+    mirror_index = "best"
     mirror = get_context().config.mirrors["pmaports"]
     for i in range(len(urls)):
         if urls[i] == mirror:
-            mirror_indexes.append(str(i + 1))
+            mirror_index = str(i + 1)
             break
-
     mirror = ""
     # require one valid mirror index selected by user
     while len(mirror) == 0:
-        answer = pmb.helpers.cli.ask(
-            "Select a mirror", None, ",".join(mirror_indexes), validation_regex=regex
-        )
-        i = int(answer)
-        if i < 1 or i > len(urls):
-            logging.info("You must select one valid mirror!")
-        mirror = urls[i - 1]
+        answer = pmb.helpers.cli.ask("Select a mirror", None, mirror_index, validation_regex=regex)
+        if answer == "best":
+            timings: list[tuple[float, str]] = []
+            # determine the best available mirror
+            for url in urls:
+                try:
+                    heapq.heappush(timings, (pmb.helpers.http.measure_latency(url), url))
+                except urllib.error.HTTPError:
+                    logging.warning(f"{url} was unavailable, skipping!")
+                    continue
+            try:
+                latency, mirror = timings[0]
+                logging.info(
+                    f"Best mirror was {mirror} with a latency of {round(latency * 1000, 2)}ms"
+                )
+            except IndexError:
+                logging.error(
+                    "No mirror was available! Please check your internet connection. Falling back to the main mirror"
+                )
+                mirror = urls[0]
+        else:
+            i = int(answer)
+            if i < 1 or i > len(urls):
+                logging.info("You must select one valid mirror!")
+            mirror = urls[i - 1]
 
     return mirror
 
@@ -668,9 +740,8 @@ def ask_for_ssh_keys(ssh_key_glob: str, default: bool) -> bool:
 
 def ask_build_pkgs_on_install(default: bool) -> bool:
     logging.info(
-        "After pmaports are changed, the binary packages may be"
-        " outdated. If you want to install postmarketOS without"
-        " changes, reply 'n' for a faster installation."
+        "After pmaports are changed, the binary packages may be outdated. If you want to install"
+        " postmarketOS without changes, reply 'n' for a faster installation."
     )
     return pmb.helpers.cli.confirm(
         "Build outdated packages during 'pmbootstrap install'?", default=default
@@ -689,9 +760,8 @@ def get_locales() -> list[str]:
 def ask_for_locale(current_locale: str) -> str:
     locales = get_locales()
     logging.info(
-        "Choose your preferred locale, like e.g. en_US. Only UTF-8"
-        " is supported, it gets appended automatically. Use"
-        " tab-completion if needed."
+        "Choose your preferred locale, like e.g. en_US. Only UTF-8 is supported, it gets appended"
+        " automatically. Use tab-completion if needed."
     )
 
     while True:
@@ -725,6 +795,8 @@ def frontend(args: PmbArgs) -> None:
     if using_default_pmaports:
         config.aports = [config.work / "cache_git/pmaports"]
 
+    config.aports[-1] = ask_for_pmaports_path(config.aports[-1])
+
     # Update args and save config (so chroots and 'pmbootstrap log' work)
     # pmb.helpers.args.update_work(args, config.work)
     pmb.config.save(args.config, config)
@@ -781,9 +853,8 @@ def frontend(args: PmbArgs) -> None:
 
     # Extra packages to be installed to rootfs
     logging.info(
-        "Additional packages that will be installed to rootfs."
-        " Specify them in a comma separated list (e.g.: vim,file)"
-        ' or "none"'
+        "Additional packages that will be installed to rootfs. Specify them in a comma separated"
+        ' list (e.g.: vim,file) or "none"'
     )
     extra = pmb.helpers.cli.ask(
         "Extra packages", None, config.extra_packages, validation_regex=r"^([-.+\w]+)(,[-.+\w]+)*$"
