@@ -1,3 +1,6 @@
+# Copyright 2017 Oliver Smith
+# SPDX-License-Identifier: GPL-3.0-or-later
+
 from pmb.core.context import get_context
 from pmb.helpers import logging
 import os
@@ -35,4 +38,4 @@ def frontend(args: PmbArgs) -> None:  # FIXME: ARGS_REFACTOR
     logging.info(f"Export symlinks to: {target}")
     if args.odin_flashable_tar:
         pmb.export.odin(config.device, flavor, target)
-    pmb.export.symlinks(flavor, target)
+    pmb.export.symlinks(target)
