@@ -78,12 +78,12 @@ def run(
 
     if no_reboot and ("flash" not in action or method != "heimdall-bootimg"):
         raise RuntimeError(
-            "The '--no-reboot' option is only" " supported when flashing with heimall-bootimg."
+            "The '--no-reboot' option is only supported when flashing with heimall-bootimg."
         )
 
     if resume and ("flash" not in action or method != "heimdall-bootimg"):
         raise RuntimeError(
-            "The '--resume' option is only" " supported when flashing with heimall-bootimg."
+            "The '--resume' option is only supported when flashing with heimall-bootimg."
         )
 
     # Run the commands of each action
@@ -103,6 +103,13 @@ def run(
                     check_partition_blacklist(deviceinfo, key, value)
                     command[i] = command[i].replace(key, value)
 
+        # Remove cmdline if it's empty
+        if "--cmdline" in command:
+            i = command.index("--cmdline")
+            if i + 1 < len(command) and not command[i + 1]:
+                command.pop(i)  # "--cmdline"
+                command.pop(i)  # ""
+
         # Remove empty strings
         command = [x for x in command if x != ""]
         # Run the action
