@@ -138,15 +138,13 @@ def extract(version: str, apk_path: Path) -> None:
     os.chmod(temp_path, os.stat(temp_path).st_mode | stat.S_IEXEC)
     version_bin = pmb.helpers.run.user_output([temp_path, "--version"])
     version_bin = version_bin.split(" ")[1].split(",")[0]
-    if not version.startswith(f"{version_bin}-r"):
+    if not version.startswith(f"{version_bin}"):
         os.unlink(temp_path)
         raise RuntimeError(
-            f"Downloaded apk-tools-static-{version}.apk,"
-            " but the apk binary inside that package reports"
-            f" to be version: {version_bin}!"
-            " Looks like a downgrade attack"
-            " from a malicious server! Switch the server (-m)"
-            " and try again."
+            f"Downloaded apk-tools-static-{version}.apk, but the apk binary inside that package"
+            f" reports to be version: {version_bin}! This could potentially be caused by a"
+            " downgrade attack from a malicious server or a bug in pmbootstrap. You can try"
+            " switching servers, see https://docs.postmarketos.org/pmbootstrap/mirrors.html"
         )
 
     # Move it to the right path
