@@ -7,7 +7,7 @@ from pmb.helpers import logging
 import os
 from pathlib import Path
 import sys
-from typing import cast, Any, NoReturn
+from typing import Any, NoReturn
 
 import pmb.aportgen
 import pmb.build
@@ -19,7 +19,7 @@ import pmb.chroot.other
 import pmb.ci
 import pmb.config
 from pmb.core import Config
-from pmb.types import Env, PmbArgs, RunOutputType
+from pmb.types import Env, PmbArgs
 import pmb.export
 import pmb.flasher
 import pmb.helpers.aportupgrade
@@ -27,6 +27,7 @@ import pmb.helpers.devices
 import pmb.helpers.git
 import pmb.helpers.lint
 import pmb.helpers.logging
+import pmb.helpers.mount
 import pmb.helpers.pmaports
 import pmb.helpers.repo
 import pmb.helpers.repo_missing
@@ -118,11 +119,6 @@ def build(args: PmbArgs) -> None:
     if src and not os.path.exists(src):
         raise RuntimeError("Invalid path specified for --src: " + src)
 
-    # Ensure repo_bootstrap is done for all arches we intend to build for
-    for package in args.packages:
-        arch_package = args.arch or pmb.build.autodetect.arch(package)
-        pmb.helpers.pmaports.require_bootstrap(arch_package, f"build {package} for {arch_package}")
-
     context = get_context()
     # Build all packages
     built = pmb.build.packages(
@@ -174,7 +170,7 @@ def chroot(args: PmbArgs) -> None:
         and chroot != Chroot.native()
         and chroot.type not in [ChrootType.BUILDROOT, ChrootType.IMAGE]
     ):
-        raise RuntimeError("--user is only supported for native or" " buildroot_* chroots.")
+        raise RuntimeError("--user is only supported for native or buildroot_* chroots.")
     if args.xauth and chroot != Chroot.native():
         raise RuntimeError("--xauth is only supported for native chroot.")
 
@@ -211,18 +207,20 @@ def chroot(args: PmbArgs) -> None:
         size_reserve = 2048  # 2 GiB
         pmb.install.blockdevice.create_and_mount_image(args, size_boot, size_root, size_reserve)
 
-    pmb.helpers.apk.update_repository_list(chroot.path, user_repository=True)
+    # Bind mount in sysfs dirs to accessing USB devices (e.g. for running fastboot)
+    if args.chroot_usb:
+        for folder in pmb.config.flash_mount_bind:
+            pmb.helpers.mount.bind(folder, Chroot.native() / folder)
 
-    # TODO: Maybe this could be done better.
-    output_type = cast(RunOutputType, args.output)
+    pmb.helpers.apk.update_repository_list(chroot.path, user_repository=True)
 
     # Run the command as user/root
     if user:
         logging.info(f"({chroot}) % su pmos -c '" + " ".join(args.command) + "'")
-        pmb.chroot.user(args.command, chroot, output=output_type, env=env)
+        pmb.chroot.user(args.command, chroot, output=args.output, env=env)
     else:
         logging.info(f"({chroot}) % " + " ".join(args.command))
-        pmb.chroot.root(args.command, chroot, output=output_type, env=env)
+        pmb.chroot.root(args.command, chroot, output=args.output, env=env)
 
 
 def config(args: PmbArgs) -> None:
@@ -246,12 +244,17 @@ def config(args: PmbArgs) -> None:
             name = args.name.split(".", 1)[1]
             # Ignore mypy 'error: TypedDict key must be a string literal'.
             # Argparse already ensures 'name' is a valid Config.Mirrors key.
-            config.mirrors[name] = args.value  # type: ignore
+            if value_changed := (config.mirrors[name] != args.value):  # type: ignore
+                config.mirrors[name] = args.value  # type: ignore
         elif isinstance(getattr(Config, args.name), list):
-            setattr(config, args.name, args.value.split(","))
+            new_list = args.value.split(",")
+            if value_changed := (getattr(config, args.name, None) != new_list):
+                setattr(config, args.name, new_list)
         else:
-            setattr(config, args.name, args.value)
-        logging.info("Config changed: " + args.name + "='" + args.value + "'")
+            if value_changed := (getattr(config, args.name) != args.value):
+                setattr(config, args.name, args.value)
+        if value_changed:
+            print(f"{args.name} = {args.value}")
         pmb.config.save(args.config, config)
     elif args.name:
         if hasattr(config, args.name):
@@ -285,7 +288,13 @@ def config(args: PmbArgs) -> None:
 
 
 def repo_missing(args: PmbArgs) -> None:
-    missing = pmb.helpers.repo_missing.generate(args.arch, args.overview, args.package, args.built)
+    if args.arch is None:
+        raise AssertionError
+    if args.built:
+        logging.warning(
+            "WARNING: --built is deprecated (bpo#148: this warning is expected on build.postmarketos.org for now)"
+        )
+    missing = pmb.helpers.repo_missing.generate(args.arch)
     print(json.dumps(missing, indent=4))
 
 
@@ -298,20 +307,14 @@ def install(args: PmbArgs) -> None:
     device = config.device
     deviceinfo = pmb.parse.deviceinfo(device)
     if args.no_fde:
-        logging.warning("WARNING: --no-fde is deprecated," " as it is now the default.")
+        logging.warning("WARNING: --no-fde is deprecated, as it is now the default.")
     if args.rsync and args.full_disk_encryption:
-        raise ValueError("Installation using rsync is not compatible with full" " disk encryption.")
+        raise ValueError("Installation using rsync is not compatible with full disk encryption.")
     if args.rsync and not args.disk:
         raise ValueError("Installation using rsync only works with --disk.")
 
     if args.rsync and args.filesystem == "btrfs":
-        raise ValueError(
-            "Installation using rsync" " is not currently supported on btrfs filesystem."
-        )
-
-    pmb.helpers.pmaports.require_bootstrap(
-        deviceinfo.arch, f"do 'pmbootstrap install' for {deviceinfo.arch}" " (deviceinfo_arch)"
-    )
+        raise ValueError("Installation using rsync is not currently supported on btrfs filesystem.")
 
     # On-device installer checks
     # Note that this can't be in the mutually exclusive group that has most of
@@ -329,20 +332,20 @@ def install(args: PmbArgs) -> None:
                 " --android-recovery-zip (patches welcome)"
             )
         if args.no_image:
-            raise ValueError("--on-device-installer cannot be combined with" " --no-image")
+            raise ValueError("--on-device-installer cannot be combined with --no-image")
         if args.rsync:
-            raise ValueError("--on-device-installer cannot be combined with" " --rsync")
+            raise ValueError("--on-device-installer cannot be combined with --rsync")
         if args.filesystem:
-            raise ValueError("--on-device-installer cannot be combined with" " --filesystem")
+            raise ValueError("--on-device-installer cannot be combined with --filesystem")
 
         if deviceinfo.cgpt_kpart:
-            raise ValueError("--on-device-installer cannot be used with" " ChromeOS devices")
+            raise ValueError("--on-device-installer cannot be used with ChromeOS devices")
     else:
         if args.ondev_cp:
             raise ValueError("--cp can only be combined with --ondev")
         if args.ondev_no_rootfs:
             raise ValueError(
-                "--no-rootfs can only be combined with --ondev." " Do you mean --no-image?"
+                "--no-rootfs can only be combined with --ondev. Do you mean --no-image?"
             )
     if args.ondev_no_rootfs:
         _install_ondev_verify_no_rootfs(device, args.ondev_cp)
@@ -371,7 +374,7 @@ def install(args: PmbArgs) -> None:
     # Android recovery zip related
     if args.android_recovery_zip and args.filesystem:
         raise ValueError(
-            "--android-recovery-zip cannot be combined with" " --filesystem (patches welcome)"
+            "--android-recovery-zip cannot be combined with --filesystem (patches welcome)"
         )
     if args.android_recovery_zip and args.full_disk_encryption:
         logging.info(
@@ -439,7 +442,7 @@ def newapkbuild(args: PmbArgs) -> None:
     # Sanity check: -n is only allowed with SRCURL
     if args.pkgname and not is_url:
         raise RuntimeError(
-            "You can only specify a pkgname (-n) when using" " SRCURL as last parameter."
+            "You can only specify a pkgname (-n) when using SRCURL as last parameter."
         )
 
     # Passthrough: Strings (e.g. -d "my description")
@@ -582,7 +585,7 @@ def ci(args: PmbArgs) -> None:
     scripts_selected = {}
     if args.scripts:
         if args.all:
-            raise RuntimeError("Combining --all with script names doesn't" " make sense")
+            raise RuntimeError("Combining --all with script names doesn't make sense")
         for script in args.scripts:
             if script not in scripts_available:
                 logging.error(
