@@ -76,7 +76,7 @@ def rev_parse(
     :returns: commit string like "90cd0ad84d390897efdcf881c0315747a4f3a966"
         or (with ``--abbrev-ref``): the branch name, e.g. "master"
     """
-    command = ["git", "rev-parse"] + extra_args + [revision]
+    command = ["git", "rev-parse", *extra_args, revision]
     rev = pmb.helpers.run.user_output(command, path, output="null" if silent else "log")
     return rev.rstrip()
 
@@ -128,8 +128,7 @@ def get_upstream_remote(aports: Path) -> str:
                 return line.split("\t", 1)[0]
 
     raise RuntimeError(
-        f"{name_repo}: could not find remote name for any URL '{urls}' in git"
-        f" repository: {aports}"
+        f"{name_repo}: could not find remote name for any URL '{urls}' in git repository: {aports}"
     )
 
 
@@ -220,9 +219,7 @@ def parse_channels_cfg(aports: Path) -> dict:
     try:
         cfg.read_string(stdout)
     except configparser.MissingSectionHeaderError:
-        logging.info(
-            "NOTE: fix this by fetching your pmaports.git, e.g." " with 'pmbootstrap pull'"
-        )
+        logging.info("NOTE: fix this by fetching your pmaports.git, e.g. with 'pmbootstrap pull'")
         raise RuntimeError(
             "Failed to read channels.cfg from"
             f" '{remote}/master' branch of your local"
