@@ -2,7 +2,7 @@
 # SPDX-License-Identifier: GPL-3.0-or-later
 import fcntl
 from pmb.core.context import get_context
-from pmb.types import PathString, Env, RunOutputType
+from pmb.types import PathString, Env, RunOutputType, RunReturnType
 from pmb.helpers import logging
 import os
 from pathlib import Path
@@ -72,7 +72,9 @@ def sanity_checks(
         raise RuntimeError("Can't use output_return with output: " + output)
 
 
-def background(cmd, working_dir=None):
+def background(
+    cmd: PathString | Sequence[PathString], working_dir: PathString | None = None
+) -> subprocess.Popen:
     """Run a subprocess in background and redirect its output to the log."""
     ret = subprocess.Popen(
         cmd, stdout=pmb.helpers.logging.logfd, stderr=pmb.helpers.logging.logfd, cwd=working_dir
@@ -81,7 +83,9 @@ def background(cmd, working_dir=None):
     return ret
 
 
-def pipe(cmd, working_dir=None):
+def pipe(
+    cmd: PathString | Sequence[PathString], working_dir: PathString | None = None
+) -> subprocess.Popen:
     """Run a subprocess in background and redirect its output to a pipe."""
     ret = subprocess.Popen(
         cmd,
@@ -114,6 +118,16 @@ def pipe_read(
 ) -> None: ...
 
 
+@overload
+def pipe_read(
+    process: subprocess.Popen,
+    output_to_stdout: bool = ...,
+    output_log: bool = ...,
+    output_return: bool = ...,
+    output_return_buffer: list[bytes] | None = ...,
+) -> None: ...
+
+
 def pipe_read(
     process: subprocess.Popen,
     output_to_stdout: bool = False,
@@ -156,7 +170,9 @@ def pipe_read(
         return
 
 
-def kill_process_tree(pid, ppids, sudo):
+# FIXME: The docstring claims that ppids should be a list of "process ID tuples", but in practice it
+# gets called with a list of string lists for the ppids argument.
+def kill_process_tree(pid: int | str, ppids: list[list[str]], sudo: bool) -> None:
     """Recursively kill a pid and its child processes.
 
     :param pid: process id that will be killed
@@ -173,7 +189,7 @@ def kill_process_tree(pid, ppids, sudo):
             kill_process_tree(child_pid, ppids, sudo)
 
 
-def kill_command(pid, sudo):
+def kill_command(pid: int, sudo: bool) -> None:
     """Kill a command process and recursively kill its child processes.
 
     :param pid: process id that will be killed
@@ -193,15 +209,15 @@ def kill_command(pid, sudo):
 
 
 def foreground_pipe(
-    cmd,
-    working_dir=None,
-    output_to_stdout=False,
-    output_return=False,
-    output_log=True,
-    output_timeout=True,
-    sudo=False,
-    stdin=None,
-):
+    cmd: PathString | Sequence[PathString],
+    working_dir: Path | None = None,
+    output_to_stdout: bool = False,
+    output_return: bool = False,
+    output_log: bool = True,
+    output_timeout: bool = True,
+    sudo: bool = False,
+    stdin: int | None = None,
+) -> tuple[int, str]:
     """Run a subprocess in foreground with redirected output.
 
     Optionally kill it after being silent for too long.
@@ -252,7 +268,7 @@ def foreground_pipe(
                 logging.info(
                     "Process did not write any output for " + str(timeout) + " seconds. Killing it."
                 )
-                logging.info("NOTE: The timeout can be increased with" " 'pmbootstrap -t'.")
+                logging.info("NOTE: The timeout can be increased with 'pmbootstrap -t'.")
                 kill_command(process.pid, sudo)
                 continue
 
@@ -268,18 +284,20 @@ def foreground_pipe(
     return (process.returncode, b"".join(output_buffer).decode("utf-8"))
 
 
-def foreground_tui(cmd, working_dir=None):
+def foreground_tui(
+    cmd: PathString | Sequence[PathString], working_dir: PathString | None = None
+) -> int:
     """Run a subprocess in foreground without redirecting any of its output.
 
     This is the only way text-based user interfaces (ncurses programs like
     vim, nano or the kernel's menuconfig) work properly.
     """
-    logging.debug("*** output passed to pmbootstrap stdout, not to this log" " ***")
+    logging.debug("*** output passed to pmbootstrap stdout, not to this log ***")
     process = subprocess.Popen(cmd, cwd=working_dir)
     return process.wait()
 
 
-def check_return_code(code, log_message):
+def check_return_code(code: int, log_message: str) -> None:
     """Check the return code of a command.
 
     :param code: exit code to check
@@ -292,13 +310,12 @@ def check_return_code(code, log_message):
         logging.debug("^" * 70)
         log_file = get_context().log
         logging.info(
-            "NOTE: The failed command's output is above the ^^^ line"
-            f" in the log file: {log_file}"
+            f"NOTE: The failed command's output is above the ^^^ line in the log file: {log_file}"
         )
-        raise RuntimeError(f"Command failed (exit code {str(code)}): " + log_message)
+        raise RuntimeError(f"Command failed (exit code {code}): " + log_message)
 
 
-def sudo_timer_iterate():
+def sudo_timer_iterate() -> None:
     """Run sudo -v and schedule a new timer to repeat the same."""
     if pmb.config.which_sudo() == "sudo":
         subprocess.Popen(["sudo", "-v"]).wait()
@@ -310,7 +327,7 @@ def sudo_timer_iterate():
     timer.start()
 
 
-def sudo_timer_start():
+def sudo_timer_start() -> None:
     """Start a timer to call sudo -v periodically, so that the password is only needed once."""
     if "sudo_timer_active" in pmb.helpers.other.cache:
         return
@@ -319,7 +336,7 @@ def sudo_timer_start():
     sudo_timer_iterate()
 
 
-def add_proxy_env_vars(env):
+def add_proxy_env_vars(env: Env) -> None:
     """Add proxy environment variables from host to the environment of the command we are running.
 
     :param env: dict of environment variables, it will be extended with all of the proxy env vars
@@ -329,7 +346,8 @@ def add_proxy_env_vars(env):
         "FTP_PROXY",
         "HTTPS_PROXY",
         "HTTP_PROXY",
-        "HTTP_PROXY_AUTH" "ftp_proxy",
+        "HTTP_PROXY_AUTH",
+        "ftp_proxy",
         "http_proxy",
         "https_proxy",
     ]
@@ -340,15 +358,15 @@ def add_proxy_env_vars(env):
 
 
 def core(
-    log_message,
-    cmd,
-    working_dir=None,
-    output="log",
-    output_return=False,
-    check=None,
-    sudo=False,
-    disable_timeout=False,
-):
+    log_message: str,
+    cmd: Sequence[PathString],
+    working_dir: Path | None = None,
+    output: RunOutputType = "log",
+    output_return: bool = False,
+    check: bool | None = None,
+    sudo: bool = False,
+    disable_timeout: bool = False,
+) -> RunReturnType:
     """Run a command and create a log entry.
 
     This is a low level function not meant to be used directly. Use one of the
