@@ -20,7 +20,7 @@ import pmb.config.pmaports
 from pmb.helpers.locale import get_xkb_layout
 from pmb.parse.deviceinfo import Deviceinfo
 from pmb.core import Config
-from pmb.types import PartitionLayout, PmbArgs
+from pmb.types import Env, PartitionLayout, PmbArgs
 import pmb.helpers.devices
 from pmb.helpers.mount import mount_device_rootfs
 import pmb.helpers.run
@@ -149,11 +149,15 @@ def copy_files_from_chroot(args: PmbArgs, chroot: Chroot) -> None:
         if args.verbose:
             rsync_flags += "vP"
         pmb.chroot.root(
-            ["rsync", rsync_flags, "--delete"] + folders + ["/mnt/install/"], working_dir=mountpoint
+            ["rsync", rsync_flags, "--delete", *folders, "/mnt/install/"], working_dir=mountpoint
         )
         pmb.chroot.root(["rm", "-rf", "/mnt/install/home"])
     else:
-        pmb.chroot.root(["cp", "-a"] + folders + ["/mnt/install/"], working_dir=mountpoint)
+        pmb.chroot.root(["cp", "-a", *folders, "/mnt/install/"], working_dir=mountpoint)
+
+    # Log how much space and inodes we have used
+    pmb.chroot.user(["df", "-h", "/mnt/install"])
+    pmb.chroot.user(["df", "-i", "/mnt/install"])
 
 
 def create_home_from_skel(filesystem: str, user: str) -> None:
@@ -285,7 +289,7 @@ def setup_login(args: PmbArgs, config: Config, chroot: Chroot) -> None:
                     pmb.chroot.root(["passwd", config.user], chroot, output="interactive")
                     break
                 except RuntimeError:
-                    logging.info("WARNING: Failed to set the password. Try it" " one more time.")
+                    logging.info("WARNING: Failed to set the password. Try it one more time.")
 
     # Disable root login
     if is_root_locked(chroot):
@@ -304,7 +308,13 @@ def copy_ssh_keys(config: Config) -> None:
     keys = []
     for key in glob.glob(os.path.expanduser(config.ssh_key_glob)):
         with open(key) as infile:
-            keys += infile.readlines()
+            try:
+                keys += infile.readlines()
+            except UnicodeDecodeError:
+                logging.info(
+                    f"WARNING: {key} is not a valid SSH key. "
+                    "This file will not be copied to device."
+                )
 
     if not len(keys):
         logging.info(
@@ -395,9 +405,11 @@ def setup_locale(chroot: Chroot, locale: str) -> None:
     # 10locale-pmos.sh gets sourced before 20locale.sh from
     # alpine-baselayout by /etc/profile. Since they don't override the
     # locale if it exists, it warranties we have preference
-    line = f"export LANG=${{LANG:-{shlex.quote(locale)}}}"
+    pmb.chroot.root(["sh", "-c", f"echo LANG={shlex.quote(locale)} > /etc/locale.conf"], chroot)
+    # musl-locales doesn't read from /etc/locale.conf, only from environment variables
+    # TODO: once musl implements locales (hopefully with /etc/locale.conf support) the following should be removed
     pmb.chroot.root(
-        ["sh", "-c", f"echo {shlex.quote(line)}" " > /etc/profile.d/10locale-pmos.sh"], chroot
+        ["sh", "-c", "echo source /etc/locale.conf > /etc/profile.d/10locale-pmos.sh"], chroot
     )
     # add keyboard layout related to locale and layout switcher
     xkb_layout = get_xkb_layout(locale)
@@ -476,18 +488,6 @@ def setup_appstream(offline: bool, chroot: Chroot) -> None:
         )
 
 
-def disable_sshd(chroot: Chroot) -> None:
-    # check=False: rc-update doesn't exit with 0 if already disabled
-    pmb.chroot.root(["rc-update", "del", "sshd", "default"], chroot, check=False)
-
-    # Verify that it's gone
-    sshd_files = pmb.helpers.run.root(
-        ["find", "-name", "sshd"], output_return=True, working_dir=chroot / "etc/runlevels"
-    )
-    if sshd_files:
-        raise RuntimeError(f"Failed to disable sshd service: {sshd_files}")
-
-
 def print_sshd_info(args: PmbArgs) -> None:
     logging.info("")  # make the note stand out
     logging.info("*** SSH DAEMON INFORMATION ***")
@@ -507,21 +507,49 @@ def print_sshd_info(args: PmbArgs) -> None:
         # on the network by default (e.g. Raspberry Pi), one can lock down the
         # installer OS down by disabling the debug user (see wiki page).
         logging.info(
-            "SSH daemon is enabled in the installer OS, to allow" " debugging the installer image."
+            "SSH daemon is enabled in the installer OS, to allow debugging the installer image."
         )
         logging.info("More info: https://postmarketos.org/ondev-debug")
 
 
-def disable_firewall(chroot: Chroot) -> None:
+def disable_service_systemd(chroot: Chroot, service_name: str) -> None:
+    preset = f"/usr/lib/systemd/system-preset/80-pmbootstrap-install-disable-{service_name}.preset"
+    preset_content = f"disable {service_name}.service"
+    pmb.chroot.root(
+        ["sh", "-c", f"echo {shlex.quote(preset_content)} > {shlex.quote(preset)}"], chroot
+    )
+    pmb.chroot.root(["systemctl", "preset", f"{service_name}.service"], chroot)
+
+    # Use the output instead of exit code to be more explicit and so we can put
+    # it in the error message
+    output = pmb.chroot.root(
+        ["systemctl", "is-enabled", f"{service_name}.service"],
+        chroot,
+        output_return=True,
+        check=False,
+    )
+    output = output.rstrip()
+    if output != "disabled":
+        raise RuntimeError(f"Failed to disable {service_name} service (systemd preset): {output}")
+
+
+def disable_service_openrc(chroot: Chroot, service_name: str) -> None:
     # check=False: rc-update doesn't exit with 0 if already disabled
-    pmb.chroot.root(["rc-update", "del", "nftables", "default"], chroot, check=False)
+    pmb.chroot.root(["rc-update", "del", service_name, "default"], chroot, check=False)
 
     # Verify that it's gone
-    nftables_files = pmb.helpers.run.root(
-        ["find", "-name", "nftables"], output_return=True, working_dir=chroot / "etc/runlevels"
+    runlevel_files = pmb.helpers.run.root(
+        ["find", "-name", service_name], output_return=True, working_dir=chroot / "etc/runlevels"
     )
-    if nftables_files:
-        raise RuntimeError(f"Failed to disable firewall: {nftables_files}")
+    if runlevel_files:
+        raise RuntimeError(f"Failed to disable service {service_name} (openrc): {runlevel_files}")
+
+
+def disable_service(chroot: Chroot, service_name: str) -> None:
+    if pmb.config.is_systemd_selected():
+        disable_service_systemd(chroot, service_name)
+    else:
+        disable_service_openrc(chroot, service_name)
 
 
 def print_firewall_info(disabled: bool, arch: Arch) -> None:
@@ -545,7 +573,7 @@ def print_firewall_info(disabled: bool, arch: Arch) -> None:
     logging.info("*** FIREWALL INFORMATION ***")
 
     if not pmaports_ok:
-        logging.info("Firewall is not supported in checked out pmaports" " branch.")
+        logging.info("Firewall is not supported in checked out pmaports branch.")
     elif disabled:
         logging.info("Firewall is disabled (--no-firewall).")
     elif not apkbuild_found:
@@ -557,9 +585,9 @@ def print_firewall_info(disabled: bool, arch: Arch) -> None:
         logging.info("Firewall is enabled and supported by kernel.")
     else:
         logging.info(
-            "Firewall is enabled, but will not work (no support in" " kernel config for nftables)."
+            "Firewall is enabled, but will not work (no support in kernel config for nftables)."
         )
-        logging.info("If/when the kernel supports it in the future, it" " will work automatically.")
+        logging.info("If/when the kernel supports it in the future, it will work automatically.")
 
     logging.info("For more information: https://postmarketos.org/firewall")
 
@@ -583,7 +611,7 @@ def generate_binary_list(args: PmbArgs, chroot: Chroot, step: int) -> list[tuple
         try:
             offset = int(_offset)
         except ValueError:
-            raise RuntimeError("Value for firmware binary offset is " f"not valid: {offset}")
+            raise RuntimeError(f"Value for firmware binary offset is not valid: {offset}")
         binary_path = chroot / "usr/share" / binary
         if not os.path.exists(binary_path):
             raise RuntimeError(
@@ -598,8 +626,7 @@ def generate_binary_list(args: PmbArgs, chroot: Chroot, step: int) -> list[tuple
         binary_size = os.path.getsize(binary_path)
         if binary_size > max_size:
             raise RuntimeError(
-                "The firmware is too big to embed in the "
-                f"disk image {binary_size}B > {max_size}B"
+                f"The firmware is too big to embed in the disk image {binary_size}B > {max_size}B"
             )
         # Insure that the firmware does not conflict with any other firmware
         # that will be embedded
@@ -610,7 +637,7 @@ def generate_binary_list(args: PmbArgs, chroot: Chroot, step: int) -> list[tuple
                 binary_end > start and binary_end <= end
             ):
                 raise RuntimeError(
-                    "The firmware overlaps with at least one " f"other firmware image: {binary}"
+                    f"The firmware overlaps with at least one other firmware image: {binary}"
                 )
 
         binary_ranges[binary_start] = binary_end
@@ -639,7 +666,7 @@ def embed_firmware(args: PmbArgs, suffix: Chroot) -> None:
             step = int(pmb.parse.deviceinfo().sd_embed_firmware_step_size or "invalid")
         except ValueError:
             raise RuntimeError(
-                "Value for " "deviceinfo_sd_embed_firmware_step_size " f"is not valid: {step}"
+                f"Value for deviceinfo_sd_embed_firmware_step_size is not valid: {step}"
             )
 
     device_rootfs = mount_device_rootfs(suffix)
@@ -649,8 +676,7 @@ def embed_firmware(args: PmbArgs, suffix: Chroot) -> None:
     for binary, offset in binary_list:
         binary_file = os.path.join("/usr/share", binary)
         logging.info(
-            f"Embed firmware {binary} in the SD card image at offset {offset} with"
-            f" step size {step}"
+            f"Embed firmware {binary} in the SD card image at offset {offset} with step size {step}"
         )
         filename = os.path.join(device_rootfs, binary_file.lstrip("/"))
         pmb.chroot.root(
@@ -673,7 +699,7 @@ def write_cgpt_kpart(args: PmbArgs, layout: PartitionLayout, suffix: Chroot) ->
     pmb.chroot.root(["dd", f"if={filename}", f"of=/dev/installp{layout['kernel']}"])
 
 
-def sanity_check_boot_size():
+def sanity_check_boot_size() -> None:
     default = Config().boot_size
     config = get_context().config
     if int(config.boot_size) >= int(default):
@@ -683,9 +709,7 @@ def sanity_check_boot_size():
         f" {config.boot_size} configured, probably because the config"
         " has been created with an old version."
     )
-    logging.error(
-        "This can lead to problems later on, we recommend setting it" f" to {default} MiB."
-    )
+    logging.error(f"This can lead to problems later on, we recommend setting it to {default} MiB.")
     logging.error(f"Run 'pmbootstrap config boot_size {default}' and try again.")
     sys.exit(1)
 
@@ -792,27 +816,30 @@ def get_uuid(args: PmbArgs, partition: Path) -> str:
     ).rstrip()
 
 
-def create_crypttab(args: PmbArgs, layout: PartitionLayout, chroot: Chroot) -> None:
+def create_crypttab(args: PmbArgs, layout: PartitionLayout | None, chroot: Chroot) -> None:
     """
     Create /etc/crypttab config
 
-    :param layout: partition layout from get_partition_layout()
+    :param layout: partition layout from get_partition_layout() or None
     :param suffix: of the chroot, which crypttab will be created to
     """
+    if layout:
+        root_dev = Path(f"/dev/installp{layout['root']}")
+    else:
+        root_dev = Path("/dev/install")
 
-    luks_uuid = get_uuid(args, Path("/dev") / f"installp{layout['root']}")
-
+    luks_uuid = get_uuid(args, root_dev)
     crypttab = f"root UUID={luks_uuid} none luks\n"
 
     (chroot / "tmp/crypttab").open("w").write(crypttab)
     pmb.chroot.root(["mv", "/tmp/crypttab", "/etc/crypttab"], chroot)
 
 
-def create_fstab(args: PmbArgs, layout: PartitionLayout, chroot: Chroot) -> None:
+def create_fstab(args: PmbArgs, layout: PartitionLayout | None, chroot: Chroot) -> None:
     """
     Create /etc/fstab config
 
-    :param layout: partition layout from get_partition_layout()
+    :param layout: partition layout from get_partition_layout() or None
     :param chroot: of the chroot, which fstab will be created to
     """
 
@@ -821,19 +848,16 @@ def create_fstab(args: PmbArgs, layout: PartitionLayout, chroot: Chroot) -> None
     if args.on_device_installer and chroot.type == ChrootType.ROOTFS:
         return
 
-    boot_dev = Path(f"/dev/installp{layout['boot']}")
-    root_dev = Path(f"/dev/installp{layout['root']}")
+    if layout:
+        boot_dev = Path(f"/dev/installp{layout['boot']}")
+        root_dev = Path(f"/dev/installp{layout['root']}")
+    else:
+        boot_dev = None
+        root_dev = Path("/dev/install")
 
-    boot_mount_point = f"UUID={get_uuid(args, boot_dev)}"
     root_mount_point = (
         "/dev/mapper/root" if args.full_disk_encryption else f"UUID={get_uuid(args, root_dev)}"
     )
-
-    boot_options = "nodev,nosuid,noexec"
-    boot_filesystem = pmb.parse.deviceinfo().boot_filesystem or "ext2"
-    if boot_filesystem in ("fat16", "fat32"):
-        boot_filesystem = "vfat"
-        boot_options += ",umask=0077,nosymfollow,codepage=437,iocharset=ascii"
     root_filesystem = pmb.install.get_root_filesystem(args)
 
     if root_filesystem == "btrfs":
@@ -846,17 +870,23 @@ def create_fstab(args: PmbArgs, layout: PartitionLayout, chroot: Chroot) -> None
 {root_mount_point} /srv btrfs subvol=@srv,compress=zstd:2,ssd 0 0
 {root_mount_point} /var btrfs subvol=@var,ssd 0 0
 {root_mount_point} /.snapshots btrfs subvol=@snapshots,compress=zstd:2,ssd 0 0
-
-{boot_mount_point} /boot {boot_filesystem} {boot_options} 0 0
 """.lstrip()
 
     else:
         fstab = f"""
 # <file system> <mount point> <type> <options> <dump> <pass>
 {root_mount_point} / {root_filesystem} defaults 0 0
-{boot_mount_point} /boot {boot_filesystem} {boot_options} 0 0
 """.lstrip()
 
+    if boot_dev:
+        boot_mount_point = f"UUID={get_uuid(args, boot_dev)}"
+        boot_options = "nodev,nosuid,noexec"
+        boot_filesystem = pmb.parse.deviceinfo().boot_filesystem or "ext2"
+        if boot_filesystem in ("fat16", "fat32"):
+            boot_filesystem = "vfat"
+            boot_options += ",umask=0077,nosymfollow,codepage=437,iocharset=ascii"
+        fstab += f"{boot_mount_point} /boot {boot_filesystem} {boot_options} 0 0\n"
+
     with (chroot / "tmp/fstab").open("w") as f:
         f.write(fstab)
     pmb.chroot.root(["mv", "/tmp/fstab", "/etc/fstab"], chroot)
@@ -871,6 +901,7 @@ def install_system_image(
     boot_label: str = "pmOS_boot",
     root_label: str = "pmOS_root",
     split: bool = False,
+    single_partition: bool = False,
     disk: Path | None = None,
 ) -> None:
     """
@@ -890,12 +921,15 @@ def install_system_image(
     logging.info(f"*** ({step}/{steps}) PREPARE INSTALL BLOCKDEVICE ***")
     pmb.helpers.mount.umount_all(chroot.path)
     (size_boot, size_root) = get_subpartitions_size(chroot)
-    layout = get_partition_layout(
-        size_reserve, bool(pmb.parse.deviceinfo().cgpt_kpart and args.install_cgpt)
-    )
+    if not single_partition:
+        layout = get_partition_layout(
+            size_reserve, bool(pmb.parse.deviceinfo().cgpt_kpart and args.install_cgpt)
+        )
+    else:
+        layout = None
     if not args.rsync:
         pmb.install.blockdevice.create(args, size_boot, size_root, size_reserve, split, disk)
-        if not split:
+        if not split and layout:
             if pmb.parse.deviceinfo().cgpt_kpart and args.install_cgpt:
                 pmb.install.partition_cgpt(layout, size_boot, size_reserve)
             else:
@@ -904,7 +938,8 @@ def install_system_image(
     # Inform kernel about changed partition table in case parted couldn't
     pmb.chroot.root(["partprobe", "/dev/install"], check=False)
 
-    if not split:
+    if not split and not single_partition:
+        assert layout  # Initialized above for not single_partition case (mypy needs this)
         pmb.install.partitions_mount(device, layout, disk)
 
     pmb.install.format(args, layout, boot_label, root_label, disk)
@@ -937,12 +972,13 @@ def install_system_image(
 
     # Don't try to embed firmware and cgpt on split images since there's no
     # place to put it and it will end up in /dev of the chroot instead
-    if not split:
+    if not split and not single_partition:
+        assert layout  # Initialized above for not single_partition case (mypy needs this)
         embed_firmware(args, chroot)
         write_cgpt_kpart(args, layout, chroot)
 
     if disk:
-        logging.info(f"Unmounting disk {disk} (this may take a while " "to sync, please wait)")
+        logging.info(f"Unmounting disk {disk} (this may take a while to sync, please wait)")
     pmb.chroot.shutdown(True)
 
     # Convert rootfs to sparse using img2simg
@@ -980,7 +1016,9 @@ def install_system_image(
             pmb.chroot.user(["mv", "-f", sys_image_patched, sys_image], working_dir=workdir)
 
 
-def print_flash_info(device: str, deviceinfo: Deviceinfo, split: bool, have_disk: bool) -> None:
+def print_flash_info(
+    device: str, deviceinfo: Deviceinfo, split: bool, have_disk: bool, single_partition: bool
+) -> None:
     """Print flashing information, based on the deviceinfo data and the
     pmbootstrap arguments."""
     logging.info("")  # make the note stand out
@@ -999,10 +1037,10 @@ def print_flash_info(device: str, deviceinfo: Deviceinfo, split: bool, have_disk
             "Refer to the installation instructions of your device,"
             " or the generic install instructions in the wiki."
         )
-        logging.info("https://wiki.postmarketos.org/wiki/Installation_guide" "#pmbootstrap_flash")
+        logging.info("https://wiki.postmarketos.org/wiki/Installation_guide#pmbootstrap_flash")
         return
 
-    logging.info("Run the following to flash your installation to the" " target device:")
+    logging.info("Run the following to flash your installation to the target device:")
 
     if "flash_rootfs" in flasher_actions and not have_disk and bool(split) == requires_split:
         logging.info("* pmbootstrap flasher flash_rootfs")
@@ -1011,12 +1049,13 @@ def print_flash_info(device: str, deviceinfo: Deviceinfo, split: bool, have_disk
             logging.info(f"  {Chroot.native() / 'home/pmos/rootfs' / device}-root.img")
         else:
             logging.info(f"  {Chroot.native() / 'home/pmos/rootfs' / device}.img")
-            logging.info(
-                "  (NOTE: This file has a partition table, which"
-                " contains /boot and / subpartitions. That way we"
-                " don't need to change the partition layout on your"
-                " device.)"
-            )
+            if not single_partition:
+                logging.info(
+                    "  (NOTE: This file has a partition table, which"
+                    " contains /boot and / subpartitions. That way we"
+                    " don't need to change the partition layout on your"
+                    " device.)"
+                )
 
     # if current flasher supports vbmeta and partition is explicitly specified
     # in deviceinfo
@@ -1101,11 +1140,12 @@ def install_on_device_installer(args: PmbArgs, step: int, steps: int) -> None:
     # Prepare the installer chroot
     logging.info(f"*** ({step}/{steps}) CREATE ON-DEVICE INSTALLER ROOTFS ***")
     step += 1
-    packages = (
-        [f"device-{config.device}", "postmarketos-ondev"]
-        + get_kernel_package(config)
-        + get_nonfree_packages(config.device)
-    )
+    packages = [
+        f"device-{config.device}",
+        "postmarketos-ondev",
+        *get_kernel_package(config),
+        *get_nonfree_packages(config.device),
+    ]
 
     chroot_installer = Chroot(ChrootType.INSTALLER, config.device)
     pmb.chroot.apk.install(packages, chroot_installer)
@@ -1126,7 +1166,7 @@ def install_on_device_installer(args: PmbArgs, step: int, steps: int) -> None:
     logging.info(f"({chroot_installer}) ondev-prepare")
     channel = pmb.config.pmaports.read_config()["channel"]
     channel_cfg = pmb.config.pmaports.read_config_channel()
-    env = {
+    env: Env = {
         "ONDEV_CHANNEL": channel,
         "ONDEV_CHANNEL_BRANCH_APORTS": channel_cfg["branch_aports"],
         "ONDEV_CHANNEL_BRANCH_PMAPORTS": channel_cfg["branch_pmaports"],
@@ -1142,7 +1182,7 @@ def install_on_device_installer(args: PmbArgs, step: int, steps: int) -> None:
     if args.ondev_cp:
         for host_src, chroot_dest in args.ondev_cp:
             host_dest = chroot_installer / chroot_dest
-            logging.info(f"({chroot_installer}) add {host_src} as" f" {chroot_dest}")
+            logging.info(f"({chroot_installer}) add {host_src} as {chroot_dest}")
             pmb.helpers.run.root(["install", "-Dm644", host_src, host_dest])
 
     # Remove $DEVICE-boot.img (we will generate a new one if --split was
@@ -1168,6 +1208,7 @@ def install_on_device_installer(args: PmbArgs, step: int, steps: int) -> None:
         boot_label,
         "pmOS_install",
         args.split,
+        args.single_partition,
         args.disk,
     )
 
@@ -1203,7 +1244,7 @@ def get_selected_providers(args: PmbArgs, packages: list[str]) -> list[str]:
         for select in apkbuild["_pmb_select"]:
             if select in get_context().config.providers:
                 ret += [get_context().config.providers[select]]
-                logging.verbose(f"{package}: install selected_providers:" f" {', '.join(ret)}")
+                logging.verbose(f"{package}: install selected_providers: {', '.join(ret)}")
             else:
                 for default in apkbuild["_pmb_default"]:
                     # default: e.g. "postmarketos-base-ui-audio-pipewire"
@@ -1263,7 +1304,7 @@ def get_recommends(args: PmbArgs, packages: list[str]) -> Sequence[str]:
                 continue
         recommends = apkbuild["_pmb_recommends"]
         if recommends:
-            logging.debug(f"{package}: install _pmb_recommends:" f" {', '.join(recommends)}")
+            logging.debug(f"{package}: install _pmb_recommends: {', '.join(recommends)}")
             ret += recommends
             # Call recursively in case recommends have pmb_recommends of their
             # own.
@@ -1282,7 +1323,7 @@ def create_device_rootfs(args: PmbArgs, step: int, steps: int) -> None:
     context = get_context()
     config = context.config
     device = context.config.device
-    logging.info(f'*** ({step}/{steps}) CREATE DEVICE ROOTFS ("{device}")' " ***")
+    logging.info(f'*** ({step}/{steps}) CREATE DEVICE ROOTFS ("{device}") ***')
 
     chroot = Chroot(ChrootType.ROOTFS, device)
     pmb.chroot.init(chroot)
@@ -1291,11 +1332,12 @@ def create_device_rootfs(args: PmbArgs, step: int, steps: int) -> None:
     set_user(context.config)
 
     # Fill install_packages
-    install_packages = pmb.config.install_device_packages + ["device-" + device]
+    install_packages = [*pmb.config.install_device_packages, "device-" + device]
     if not args.install_base:
         install_packages = [p for p in install_packages if p != "postmarketos-base"]
+    ui_package_name = f"postmarketos-ui-{config.ui}"
     if config.ui.lower() != "none":
-        install_packages += ["postmarketos-ui-" + config.ui]
+        install_packages += [ui_package_name]
 
     # Add additional providers of base/device/UI package
     install_packages += get_selected_providers(args, install_packages)
@@ -1303,15 +1345,17 @@ def create_device_rootfs(args: PmbArgs, step: int, steps: int) -> None:
     install_packages += get_kernel_package(config)
     install_packages += get_nonfree_packages(device)
     if context.config.ui.lower() != "none":
-        if context.config.ui_extras:
-            install_packages += ["postmarketos-ui-" + config.ui + "-extras"]
+        ui_package = pmb.helpers.pmaports.get(ui_package_name, subpackages=False, must_exist=False)
+        if ui_package and context.config.ui_extras:
+            extra = f"postmarketos-ui-{config.ui}-extras"
+            extra_package = ui_package["subpackages"].get(extra)
+            if extra_package:
+                install_packages += [extra]
+
     if context.config.extra_packages.lower() != "none":
         install_packages += context.config.extra_packages.split(",")
     if args.add:
         install_packages += args.add.split(",")
-    locale_is_set = config.locale != Config().locale
-    if locale_is_set:
-        install_packages += ["lang", "musl-locales"]
 
     pmaports_cfg = pmb.config.pmaports.read_config()
     # postmarketos-base supports a dummy package for blocking unl0kr install
@@ -1364,8 +1408,7 @@ def create_device_rootfs(args: PmbArgs, step: int, steps: int) -> None:
     setup_timezone(chroot, config.timezone)
 
     # Set locale
-    if locale_is_set:
-        setup_locale(chroot, config.locale)
+    setup_locale(chroot, config.locale)
 
     # Set the hostname as the device name
     setup_hostname(device, config.hostname)
@@ -1373,9 +1416,9 @@ def create_device_rootfs(args: PmbArgs, step: int, steps: int) -> None:
     setup_appstream(context.offline, chroot)
 
     if args.no_sshd:
-        disable_sshd(chroot)
+        disable_service(chroot, "sshd")
     if args.no_firewall:
-        disable_firewall(chroot)
+        disable_service(chroot, "nftables")
 
 
 def install(args: PmbArgs) -> None:
@@ -1390,6 +1433,13 @@ def install(args: PmbArgs) -> None:
     if args.on_device_installer:
         sanity_check_ondev_version(args)
 
+    # --single-partition implies --no-split. There is nothing to split if
+    # there is only a single partition.
+    if args.single_partition:
+        args.split = False
+        if deviceinfo.create_initfs_extra:
+            raise RuntimeError("--single-partition does not work for devices with initramfs-extra")
+
     # Number of steps for the different installation methods.
     if args.no_image:
         steps = 2
@@ -1423,10 +1473,23 @@ def install(args: PmbArgs) -> None:
         # Runs install_system_image twice
         install_on_device_installer(args, step, steps)
     else:
-        install_system_image(args, 0, chroot, step, steps, split=args.split, disk=args.disk)
+        install_system_image(
+            args,
+            0,
+            chroot,
+            step,
+            steps,
+            split=args.split,
+            disk=args.disk,
+            single_partition=args.single_partition,
+        )
 
     print_flash_info(
-        device, deviceinfo, args.split, True if args.disk and args.disk.is_absolute() else False
+        device,
+        deviceinfo,
+        args.split,
+        True if args.disk and args.disk.is_absolute() else False,
+        args.single_partition,
     )
     print_sshd_info(args)
     print_firewall_info(args.no_firewall, deviceinfo.arch)
