@@ -70,7 +70,7 @@ def parse_next_block(path: Path, lines: list[str]) -> ApkindexBlock | None:
     if required_found != len(required_apkindex_keys):
         for key in required_apkindex_keys:
             if key not in ret:
-                raise RuntimeError(f"Missing required key '{key}' in block " f"{ret}, file: {path}")
+                raise RuntimeError(f"Missing required key '{key}' in block {ret}, file: {path}")
         raise RuntimeError(
             f"Expected {len(required_apkindex_keys)} required keys,"
             f" but found {required_found} in block: {ret}, file: {path}"
@@ -117,7 +117,7 @@ def parse_add_block(
     ret: dict[str, ApkindexBlock],
     block: ApkindexBlock,
     alias: str | None = ...,
-    multiple_providers: bool = ...,  # FIXME: Type should be Literal[False], but mypy complains?
+    multiple_providers: Literal[False] = ...,
 ) -> None: ...
 
 
@@ -130,6 +130,15 @@ def parse_add_block(
 ) -> None: ...
 
 
+@overload
+def parse_add_block(
+    ret: dict[str, ApkindexBlock] | dict[str, dict[str, ApkindexBlock]],
+    block: ApkindexBlock,
+    alias: str | None = ...,
+    multiple_providers: bool = ...,
+) -> None: ...
+
+
 def parse_add_block(
     ret: dict[str, ApkindexBlock] | dict[str, dict[str, ApkindexBlock]],
     block: ApkindexBlock,
@@ -274,7 +283,7 @@ def parse(
 
         # Skip virtual packages
         if block.timestamp is None:
-            logging.verbose(f"Skipped virtual package {block} in" f" file: {path}")
+            logging.verbose(f"Skipped virtual package {block} in file: {path}")
             continue
 
         # Add the next package and all aliases
@@ -474,11 +483,21 @@ def package(
     if package in package_providers:
         return package_providers[package]
 
+    if package_providers:
+        providers_priority = provider_highest_priority(package_providers, package)
+        num_providers = len(providers_priority)
+        # Only one provider with max priority: return it
+        if num_providers == 1:
+            return next(iter(providers_priority.values()))
+        # Multiple providers with max priority: let follow-on logic decide
+        elif num_providers > 1:
+            package_providers = providers_priority
+
     # Any provider
     if package_providers:
         return pmb.parse.apkindex.provider_shortest(package_providers, package)
 
     # No provider
     if must_exist:
-        raise RuntimeError("Package '" + package + "' not found in any" " APKINDEX.")
+        raise RuntimeError("Package '" + package + "' not found in any APKINDEX.")
     return None
