@@ -85,6 +85,7 @@ def bootimg(path: Path) -> Bootimg:
         " we can set up a chroot to extract and analyze your"
         " boot.img file"
     )
+    pmb.chroot.init(Chroot.native())
     pmb.chroot.apk.install(["file", "unpackbootimg"], Chroot.native())
 
     temp_path = Path("/tmp/bootimg_parser")
@@ -102,8 +103,7 @@ def bootimg(path: Path) -> Bootimg:
     if "android bootimg" not in file_output.lower():
         if get_context().force:
             logging.warning(
-                "WARNING: boot.img file seems to be invalid, but"
-                " proceeding anyway (-f specified)"
+                "WARNING: boot.img file seems to be invalid, but proceeding anyway (-f specified)"
             )
         else:
             logging.info(
